@@ -23,8 +23,10 @@
  */
 
 
-// KMix
+// Own
 #include "mixer_alsa9.h"
+
+// KMix
 #include "core/kmixdevicemanager.h"
 #include "core/mixer.h"
 #include "core/volume.h"
@@ -33,100 +35,53 @@
 #include <klocalizedstring.h>
 
 // Qt
-#include <qtimer.h>
-
-// STD Headers
-#include <stdlib.h>
-#include <stdio.h>
-#include <iostream>
-#include <assert.h>
 #include <qsocketnotifier.h>
 
-#ifdef HAVE_CANBERRA
-#include <canberra.h>
-#endif
-
-
-#ifdef HAVE_CANBERRA
-static ca_context *s_ccontext = nullptr;
-static unsigned int s_refcount = 0;
-#endif
-
 // #define if you want MUCH debugging output
 //#define ALSA_SWITCH_DEBUG
 //#define KMIX_ALSA_VOLUME_DEBUG
 
+// For ALSA API documentation se
+// https://www.alsa-project.org/alsa-doc/alsa-lib/index.html
 
-Mixer_Backend*
+
+MixerBackend*
 ALSA_getMixer(Mixer *mixer, int device )
 {
 
-   Mixer_Backend *l_mixer;
+   MixerBackend *l_mixer;
 
    l_mixer = new Mixer_ALSA(mixer,  device );
    return l_mixer;
 }
 
-Mixer_ALSA::Mixer_ALSA( Mixer* mixer, int device ) : Mixer_Backend(mixer,  device )
+Mixer_ALSA::Mixer_ALSA( Mixer* mixer, int device ) : MixerBackend(mixer,  device )
 {
     m_fds = 0;
     _handle = 0;
     ctl_handle = 0;
     _initialUpdate = true;
-
-#ifdef HAVE_CANBERRA
-    ++s_refcount;					// increment Canberra reference count
-    if (s_refcount==1)					// initialise Canberra the first time
-    {
-        int ret = ca_context_create(&s_ccontext);
-        if (ret<0)
-        {
-            qCWarning(KMIX_LOG) << "Failed to create Canberra context for volume feedback," << ca_strerror(ret);
-            s_ccontext = nullptr;
-            return;
-        }
-
-        ca_context_set_driver(s_ccontext, "alsa");
-        qCDebug(KMIX_LOG) << "Initialised Canberra context for volume feedback";
-    }
-    m_playFeedbackTimer = new QTimer(this);
-    m_playFeedbackTimer->setSingleShot(true);
-    m_playFeedbackTimer->setInterval(100);
-    m_playFeedbackTimer->callOnTimeout(this, &Mixer_ALSA::playFeedbackSound);
-#endif
 }
 
 
 Mixer_ALSA::~Mixer_ALSA()
 {
     close();
-
-#ifdef HAVE_CANBERRA
-    if (s_refcount>0)					// have some Canberra references
-    {
-        --s_refcount;					// decrement Canberra reference count
-        if (s_refcount==0)				// no more references remaining
-        {
-            if (s_ccontext!=nullptr)			// have a Canberra context
-            {
-                ca_context_destroy(s_ccontext);		// don't need it any more
-                s_ccontext = nullptr;
-                qCDebug(KMIX_LOG) << "Finished with Canberra context";
-            }
-        }
-    }
-#endif
 }
 
 
-int Mixer_ALSA::identify( snd_mixer_selem_id_t *sid )
+static MixDevice::ChannelType identify(snd_mixer_selem_id_t *sid)
 {
-   const char * cname = snd_mixer_selem_id_get_name( sid );
-   QByteArray name = QByteArray::fromRawData( cname, qstrlen(cname) ).toLower();
+   const char *cname = snd_mixer_selem_id_get_name(sid);
+   const QByteArray name = QByteArray::fromRawData(cname, qstrlen(cname)).toLower();
+
    if (name.contains("master"     )) return MixDevice::VOLUME;
    if (name.contains("master mono")) return MixDevice::VOLUME;
-   if (name.contains("front"      ) &&
-      !name.contains("mic"        )) return MixDevice::VOLUME;
+   if (name.contains("mic boost"  )) return MixDevice::MICROPHONE_BOOST;
+   if (name.contains("mic front"  )) return MixDevice::MICROPHONE_FRONT;
+   if (name.contains("front mic"  )) return MixDevice::MICROPHONE_FRONT;
+   if (name.contains("mic"        )) return MixDevice::MICROPHONE;
+   if (name.contains("front"      )) return MixDevice::VOLUME;
    if (name.contains("pc speaker" )) return MixDevice::SPEAKER;
    if (name.contains("capture"    )) return MixDevice::RECMONITOR;
    if (name.contains("music"      )) return MixDevice::MIDI;
@@ -146,10 +101,6 @@ int Mixer_ALSA::identify( snd_mixer_selem_id_t *sid )
    if (name.contains("optical"    )) return MixDevice::DIGITAL;
    if (name.contains("iec958"     )) return MixDevice::DIGITAL;
    if (name.contains("digital"    )) return MixDevice::DIGITAL;
-   if (name.contains("mic boost"  )) return MixDevice::MICROPHONE_BOOST;
-   if (name.contains("mic front"  )) return MixDevice::MICROPHONE_FRONT;
-   if (name.contains("front mic"  )) return MixDevice::MICROPHONE_FRONT;
-   if (name.contains("mic"        )) return MixDevice::MICROPHONE;
    if (name.contains("lfe"        )) return MixDevice::SURROUND_LFE;
    if (name.contains("monitor"    )) return MixDevice::RECMONITOR;
    if (name.contains("3d"         )) return MixDevice::SURROUND;
@@ -180,9 +131,6 @@ int Mixer_ALSA::open()
     err = openAlsaDevice(m_deviceName);
     if (err!=0) return (err);
 
-    _udi = KMixDeviceManager::instance()->getUDI_ALSA(m_devnum);
-    if (_udi.isEmpty()) qCWarning(KMIX_LOG) << "No UDI found for" << m_deviceName << "so hotplugging not possible";
-
     // Run a loop over all controls of the card
     unsigned int idx = 0;
     for ( elem = snd_mixer_first_elem( _handle ); elem; elem = snd_mixer_elem_next( elem ) )
@@ -203,7 +151,7 @@ int Mixer_ALSA::open()
                     .arg(snd_mixer_selem_id_get_index( sid ) );
         mdID.replace(' ','_'); // Any key/ID we use, must not uses spaces (rule)
 
-        const MixDevice::ChannelType ct = static_cast<MixDevice::ChannelType>(identify(sid));
+        const MixDevice::ChannelType ct = identify(sid);
 
         /* ------------------------------------------------------------------------------- */
 
@@ -249,8 +197,10 @@ int Mixer_ALSA::open()
         mixer_sid_list.append( sid );
         idx++;
 
-
-        MixDevice* mdNew = new MixDevice(_mixer, finalMixdeviceID, readableName, ct );
+        MixDevice *mdNew = new MixDevice(_mixer, finalMixdeviceID, readableName, ct );
+        mdNew->setHardwareId("hw:"+QByteArray::number(m_devnum));
+        // TODO: this does not appear to work with volume feedback
+        //mdNew->setHardwareId("hw:"+QByteArray::number(m_devnum)+","+QByteArray::number(idx));
 
         if ( volPlay    != 0      )
         {
@@ -511,17 +461,15 @@ Volume* Mixer_ALSA::addVolume(snd_mixer_elem_t *elem, bool capture)
        snd_mixer_selem_get_capture_volume_range( elem, &minVolume, &maxVolume );
     }
 
-
     // Check if this control has at least one volume control
     bool hasVolume = snd_mixer_selem_has_playback_volume(elem) || snd_mixer_selem_has_capture_volume(elem);
 
     // Check if a appropriate switch is present (appropriate means, based o nthe "capture" parameter)
     bool hasCommonSwitch = snd_mixer_selem_has_common_switch ( elem );
 
-    bool hasSwitch = hasCommonSwitch |
-        capture
-        ? snd_mixer_selem_has_capture_switch ( elem )
-        : snd_mixer_selem_has_playback_switch ( elem );
+    bool hasSwitch = hasCommonSwitch ||
+                     (capture ? snd_mixer_selem_has_capture_switch(elem)
+                              : snd_mixer_selem_has_playback_switch(elem));
 
     if ( hasVolume || hasSwitch ) {
         //qCDebug(KMIX_LOG) << "Add somthing with chn=" << chn << ", capture=" << capture;
@@ -645,16 +593,13 @@ snd_mixer_elem_t* Mixer_ALSA::getMixerElem(int idx) {
  */
 }
 
-int Mixer_ALSA::id2num(const QString& id) {
-   //qCDebug(KMIX_LOG) << "id2num() id=" << id;
-   int num = -1;
-   if ( m_id2numHash.contains(id) ) {
-      num = m_id2numHash[id];
-   }
-   //qCDebug(KMIX_LOG) << "id2num() num=" << num;
-   return num;
+
+/* private */ int Mixer_ALSA::id2num(const QString& id)
+{
+   return (m_id2numHash.value(id, -1));
 }
 
+
 bool Mixer_ALSA::hasChangedControls() {
     if ( !m_fds || !m_isOpen )
         return false;
@@ -828,7 +773,7 @@ Mixer_ALSA::readVolumeFromHW( const QString& id, shared_ptr<MixDevice> md )
     	}
     	else
     	{
-        foreach (VolumeChannel vc, volumePlayback.getVolumes() )
+            for (const VolumeChannel &vc : qAsConst(volumePlayback.getVolumes()))
         {
                int ret = 0;
                switch(vc.chid) {
@@ -864,8 +809,8 @@ Mixer_ALSA::readVolumeFromHW( const QString& id, shared_ptr<MixDevice> md )
     // --- capture volume
     if ( snd_mixer_selem_has_capture_volume ( elem ) )
     {
-        foreach (VolumeChannel vc, volumeCapture.getVolumes() )
-		{
+        for (const VolumeChannel &vc : qAsConst(volumeCapture.getVolumes()))
+        {
                int ret = 0;
                switch(vc.chid) {
                    case Volume::LEFT         : ret = snd_mixer_selem_get_capture_volume( elem, SND_MIXER_SCHN_FRONT_LEFT  , &vol); break;
@@ -895,7 +840,7 @@ Mixer_ALSA::readVolumeFromHW( const QString& id, shared_ptr<MixDevice> md )
         // Refresh the capture switch information of *all* controls of this card.
         // Doing it for all is necessary, because enabling one record source often
         // automatically disables another record source (due to the hardware design)
-        foreach ( shared_ptr<MixDevice> md, m_mixDevices )
+        for (const shared_ptr<MixDevice> &md : qAsConst(m_mixDevices))
         {
             bool isRecsrc =  isRecsrcHW( md->id() );
             // qCDebug(KMIX_LOG) << "Mixer::setRecordSource(): isRecsrcHW(" <<  md->id() << ") =" <<  isRecsrc;
@@ -944,7 +889,7 @@ Mixer_ALSA::writeVolumeToHW( const QString& id, shared_ptr<MixDevice> md )
     	}
     	else
     	{
-      foreach (VolumeChannel vc, volumePlayback.getVolumes() )
+            for (const VolumeChannel &vc : qAsConst(volumePlayback.getVolumes()))
       {
                int ret = 0;
                switch(vc.chid)
@@ -965,17 +910,12 @@ Mixer_ALSA::writeVolumeToHW( const QString& id, shared_ptr<MixDevice> md )
               //if (id== "Master:0" || id== "PCM:0" ) { qCDebug(KMIX_LOG) << "volumePlayback control=" << id << ", chid=" << vc.chid << ", vol=" << vc.volume; }
           }
     	}
-
-#ifdef HAVE_CANBERRA
-        m_playFeedbackTimer->start();
-#endif
-
     } // has playback volume
 
     // --- capture volume
     if ( snd_mixer_selem_has_capture_volume ( elem ) )
     {
-      foreach (VolumeChannel vc, volumeCapture.getVolumes() )
+        for (const VolumeChannel &vc : qAsConst(volumeCapture.getVolumes()))
       {
                int ret = 0;
                switch(vc.chid) {
@@ -1025,70 +965,41 @@ Mixer_ALSA::errorText( int mixer_error )
 					"soundcard driver is loaded.\n" );
 			break;
 		default:
-			l_s_errmsg = Mixer_Backend::errorText( mixer_error );
+			l_s_errmsg = MixerBackend::errorText( mixer_error );
 	}
 	return l_s_errmsg;
 }
 
 
-QString
-ALSA_getDriverName()
-{
-	return QStringLiteral("ALSA");
-}
+const char *ALSA_driverName = "ALSA";
 
 QString Mixer_ALSA::getDriverName()
 {
-	return QStringLiteral("ALSA");
+    return (ALSA_driverName);
 }
 
 
-#ifdef HAVE_CANBERRA
-
-void Mixer_ALSA::playFeedbackSound()
+int ALSA_acceptsHotplugId(const QString &id)
 {
-    if (!Mixer::getBeepOnVolumeChange()) return;	// no feedback sound required
-    if (s_ccontext==nullptr) return;			// Canberra not set up
+    // The Solid device UDIs for a plugged ALSA sound card are of the form:
+    //
+    //   /org/kde/solid/udev/sys/devices/pci0000:00/0000:00:13.2/usb2/2-4/2-4.1/2-4.1:1.0/sound/card3
+    //   /org/kde/solid/udev/sys/devices/pci0000:00/0000:00:13.2/usb2/2-4/2-4.1/2-4.1:1.0/sound/card3/pcmC3D0p
+    //   /org/kde/solid/udev/sys/devices/pci0000:00/0000:00:13.2/usb2/2-1/2-1.3/2-1.3:1.0/sound/card3/controlC3
+    //
+    // The "control" one of these is taken as the canonical form.  The "card" and
+    // "control" numbers are assumed to be the same, but this is not checked.
 
-    int playing = 0;
-    // Note that '2' is simply an index we've picked.
-    // It's mostly irrelevant.
-    const int cindex = 2;
-
-    ca_context_playing(s_ccontext, cindex, &playing);
-    // Note: Depending on how this is desired to work,
-    // we may want to simply skip playing, or cancel the
-    // currently playing sound and play our
-    // new one... for now, let's do the latter.
-    if (playing!=0)
-    {
-        ca_context_cancel(s_ccontext, cindex);
-        playing = 0;
-    }
-
-    if (playing==0)
-    {
-        if (!m_deviceName.isEmpty()) ca_context_change_device(s_ccontext, m_deviceName.constData());
-
-        // Ideally we'd use something like ca_gtk_play_for_widget()...
-        int ret = ca_context_play(s_ccontext,
-                                  cindex,
-                                  CA_PROP_EVENT_DESCRIPTION, i18n("Volume Control Feedback Sound").toUtf8().constData(),
-                                  CA_PROP_EVENT_ID, "audio-volume-change",
-                                  CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
-                                  CA_PROP_CANBERRA_ENABLE, "1",
-                                  nullptr);
-
-        // Sometimes trying to play sounds in quick succession returns the
-        // error CA_ERROR_NOTAVAILABLE = "Not available", even though playing
-        // the previous sound has been cancelled above.  Ignore that error.
-        if (ret<0 && ret!=CA_ERROR_NOTAVAILABLE)
-        {
-            qCWarning(KMIX_LOG) << "Failed to play Canberra sound for volume feedback," << ca_strerror(ret);
-        }
-
-        ca_context_change_device(s_ccontext, nullptr);
-    }
+    QRegExp rx("/card(\\d+)/controlC(\\d+)$");		// match sound card control device
+    if (!id.contains(rx)) return (-1);			// UDI not recognised
+    return (rx.cap(2).toInt());				// assume conversion succeeds
 }
 
-#endif // HAVE_CANBERRA
+
+bool ALSA_acceptsDeviceNode(const QString &blkdev, int devnum)
+{
+    // The primary ALSA device is "/dev/snd/controlC<N>" which corresponds to
+    // the "control" UDI as above.
+    const QString dev = "/dev/snd/controlC"+QString::number(devnum);
+    return (blkdev==dev);
+}
