@@ -21,7 +21,6 @@
 
 //OSS4 mixer backend for KMix by Yoper Team released under GPL v2 or later
 
-/* We're getting soundcard.h via mixer_oss4.h */
 #include "mixer_oss4.h"
 
 #include <fcntl.h>
@@ -31,17 +30,30 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_SOUNDCARD_H
+#include <soundcard.h>
+#else
+#ifdef HAVE_SYS_SOUNDCARD_H
+#include <sys/soundcard.h>
+#endif
+#endif
+
 #include <QRegExp>
 #include <qplatformdefs.h>
 
-Mixer_Backend* OSS4_getMixer(Mixer *mixer, int device)
+#include "core/mixer.h"
+
+#include <klocalizedstring.h>
+
+
+MixerBackend* OSS4_getMixer(Mixer *mixer, int device)
 {
-	Mixer_Backend *l_mixer;
+	MixerBackend *l_mixer;
 	l_mixer = new Mixer_OSS4(mixer, device);
 	return l_mixer;
 }
 
-Mixer_OSS4::Mixer_OSS4(Mixer *mixer, int device) : Mixer_Backend(mixer, device)
+Mixer_OSS4::Mixer_OSS4(Mixer *mixer, int device) : MixerBackend(mixer, device)
 {
 	if ( device == -1 ) m_devnum = 0;
 	m_numExtensions = 0;
@@ -53,11 +65,10 @@ Mixer_OSS4::Mixer_OSS4(Mixer *mixer, int device) : Mixer_Backend(mixer, device)
 bool Mixer_OSS4::CheckCapture(oss_mixext *ext)
 {
 	QString name = ext->extname;
-	if ( ext->flags & MIXF_RECVOL || name.split('.').contains("in") )
-	{
-		return true;
-	}
-	return false;
+#ifdef MIXF_RECVOL					// in 4Front only
+	if (ext->flags & MIXF_RECVOL) return true;
+#endif
+	return (name.split('.').contains("in"));
 }
 
 Mixer_OSS4::~Mixer_OSS4()
@@ -71,13 +82,14 @@ MixDevice::ChannelType Mixer_OSS4::classifyAndRename(QString &name, int flags)
 	MixDevice::ChannelType cType = MixDevice::UNKNOWN;
 	QStringList classes = name.split (QRegExp( QLatin1String(  "[-,.]" ) ));
 
-
-	if ( flags & MIXF_PCMVOL  ||
-	     flags & MIXF_MONVOL  ||
-	     flags & MIXF_MAINVOL )
+#ifdef MIXF_PCMVOL
+	if (flags & MIXF_PCMVOL  ||			// these flags in 4Front only
+	    flags & MIXF_MONVOL  ||
+	    flags & MIXF_MAINVOL)
 	{
 		cType = MixDevice::VOLUME;
 	}
+#endif
 
 	for ( QStringList::Iterator it = classes.begin(); it != classes.end(); ++it )
 	{
@@ -183,7 +195,8 @@ MixDevice::ChannelType Mixer_OSS4::classifyAndRename(QString &name, int flags)
 
 int Mixer_OSS4::open()
 {
-	if ( (m_fd= QT_OPEN("/dev/mixer", O_RDWR)) < 0 )
+	QByteArray devnode = "/dev/mixer";
+	if ((m_fd = QT_OPEN(devnode, O_RDWR))<0)
 	{
 		if ( errno == EACCES )
 			return Mixer::ERR_PERM;
@@ -232,8 +245,15 @@ int Mixer_OSS4::open()
 				return Mixer::ERR_READ;
 			}
 
-			::close(m_fd);
-			if ( (m_fd= QT_OPEN(mi.devnode, O_RDWR)) < 0 )
+			QT_CLOSE(m_fd);
+#ifdef HAVE_MIXERINFO_DEVNODE
+			// oss_mixerinfo.devnode is in 4Front but not in BSD.
+			devnode = mi.devnode;
+#else
+			// Assume that the device is "/dev/mixerN" as for OSS3.
+			if (m_devnum>0) devnode += QByteArray::number(m_devnum);
+#endif
+			if ((m_fd = QT_OPEN(devnode, O_RDWR))<0)
 			{
 				return Mixer::ERR_OPEN;
 			}
@@ -313,16 +333,21 @@ int Mixer_OSS4::open()
 
 				MixDevice::ChannelType cType = classifyAndRename(name, ext.flags);
 
-				if ( (ext.type == MIXT_STEREOSLIDER16 ||
-				      ext.type == MIXT_STEREOSLIDER   ||
-				      ext.type == MIXT_MONOSLIDER16   ||
-				      ext.type == MIXT_MONOSLIDER     ||
-				      ext.type == MIXT_SLIDER 
-				      ) 				   )
+				if ((ext.type == MIXT_STEREOSLIDER   ||
+				     ext.type == MIXT_MONOSLIDER     ||
+#ifdef MIXT_STEREOSLIDER16				// in 4Front only
+				     ext.type == MIXT_STEREOSLIDER16 ||
+#endif
+#ifdef MIXT_MONOSLIDER16				// in 4Front only
+				     ext.type == MIXT_MONOSLIDER16   ||
+#endif
+				     ext.type == MIXT_SLIDER))
 				{
-					if ( ext.type == MIXT_STEREOSLIDER16 ||
-					        ext.type == MIXT_STEREOSLIDER
-					   )
+					if (
+#ifdef MIXT_STEREOSLIDER16				// in 4Front only
+						ext.type == MIXT_STEREOSLIDER16 ||
+#endif
+					        ext.type == MIXT_STEREOSLIDER)
 					{
 						chMask = Volume::ChannelMask(Volume::MLEFT|Volume::MRIGHT);
 					}
@@ -356,12 +381,18 @@ int Mixer_OSS4::open()
 						masterHeuristic = md;
 						masterHeuristicAvailable = true;
 					}
+
 					
-					if ( !masterChosen && ext.flags & MIXF_MAINVOL )
+#ifdef MIXF_MAINVOL					// in 4Front only
+					// If it is not possible to choose the master
+					// channel this way, then hope that the heuristic
+					// will do that.
+					if (!masterChosen && (ext.flags & MIXF_MAINVOL))
 					{
 						m_recommendedMaster = md;
 						masterChosen = true;
 					}
+#endif
 				}
 				else if ( ext.type == MIXT_HEXVALUE )
 				{
@@ -386,14 +417,16 @@ int Mixer_OSS4::open()
 						md->addPlaybackVolume(vol);
 					}
 					
-					if ( !masterChosen && ext.flags & MIXF_MAINVOL )
+#ifdef MIXF_MAINVOL					// in 4Front only, see above
+					if (!masterChosen && (ext.flags & MIXF_MAINVOL))
 					{
 						m_recommendedMaster = md;
 						masterChosen = true;
 					}
+#endif
 				}
 				else if ( ext.type == MIXT_ONOFF 
-#ifdef MIXT_MUTE
+#ifdef MIXT_MUTE					// in 4Front only
 					|| ext.type == MIXT_MUTE
 #endif
 					)
@@ -503,7 +536,7 @@ QString Mixer_OSS4::errorText(int mixer_error)
 			                  "Use 'soundon' when using OSS4 from 4front.");
 			break;
 		default:
-			l_s_errmsg = Mixer_Backend::errorText(mixer_error);
+			l_s_errmsg = MixerBackend::errorText(mixer_error);
 	}
 	return l_s_errmsg;
 }
@@ -569,7 +602,7 @@ int Mixer_OSS4::readVolumeFromHW(const QString& id, shared_ptr<MixDevice> md)
 		
 		switch ( extinfo.type )
 		{
-#ifdef MIXT_MUTE		  
+#ifdef MIXT_MUTE		  			// in 4Front only
 			case MIXT_MUTE:
 #endif			  
 			case MIXT_ONOFF:
@@ -588,15 +621,17 @@ int Mixer_OSS4::readVolumeFromHW(const QString& id, shared_ptr<MixDevice> md)
 			case MIXT_SLIDER:
 				vol.setVolume(Volume::LEFT, mv.value);
 				break;
-
+#ifdef MIXT_MONOSLIDER16	  			// in 4Front only
 			case MIXT_MONOSLIDER16:
 				vol.setVolume(Volume::LEFT, mv.value & 0xffff);
 				break;
-
+#endif
+#ifdef MIXT_STEREOSLIDER16	  			// in 4Front only
 			case MIXT_STEREOSLIDER16:
 				vol.setVolume(Volume::LEFT, mv.value & 0xffff);
 				vol.setVolume(Volume::RIGHT, ( mv.value >> 16 ) & 0xffff);
 				break;
+#endif
 		}
 	}
 	return 0;
@@ -642,13 +677,16 @@ int Mixer_OSS4::writeVolumeToHW(const QString& id, shared_ptr<MixDevice> md)
 			volume = vol.getVolume(Volume::LEFT);
 			break;
 
+#ifdef MIXT_MONOSLIDER16				// in 4Front only
 		case MIXT_MONOSLIDER16:
 			volume = vol.getVolume(Volume::LEFT);
 			break;
-
+#endif
+#ifdef MIXT_STEREOSLIDER16				// in 4Front only
 		case MIXT_STEREOSLIDER16:
 			volume = vol.getVolume(Volume::LEFT) | ( vol.getVolume(Volume::RIGHT) << 16 );
 			break;
+#endif
 		default:
 			return -1;
 	}
@@ -786,13 +824,10 @@ int Mixer_OSS4::wrapIoctl(int ioctlRet)
 	return ioctlRet;
 }
 
+
+const char *OSS4_driverName = "OSS4";
+
 QString Mixer_OSS4::getDriverName()
 {
-	return "OSS4";
+	return (OSS4_driverName);
 }
-
-QString OSS4_getDriverName()
-{
-	return "OSS4";
-}
-
